@@ -1,5 +1,5 @@
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2001 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -7,6 +7,7 @@
 #include	"authmod.h"
 #include	"authstaticlist.h"
 #include	"authsasl.h"
+#include	"soxwrap/sconnect.h"
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
@@ -28,15 +29,12 @@
 
 #define	TIMEOUT		15
 
-static const char rcsid[]="$Id: authdaemonlib.c,v 1.2 2001/03/23 03:22:11 mrsam Exp $";
+static const char rcsid[]="$Id: authdaemonlib.c,v 1.4 2001/12/08 21:24:50 mrsam Exp $";
 
 static int opensock()
 {
 int	s=socket(PF_UNIX, SOCK_STREAM, 0);
 struct  sockaddr_un skun;
-fd_set	fdr;
-struct timeval tv;
-int	rc;
 
 	skun.sun_family=AF_UNIX;
 	strcpy(skun.sun_path, AUTHDAEMONSOCK);
@@ -47,60 +45,10 @@ int	rc;
 		return (-1);
 	}
 
-        if (fcntl(s, F_SETFL, O_NONBLOCK) < 0)
+	if (s_connect(s, (const struct sockaddr *)&skun, sizeof(skun),
+		      TIMEOUT, 0))
 	{
-		syslog(LOG_CRIT, "authdaemon: fcntl() failed: %m");
-		close(s);
-		return (-1);
-	}
-
-	if ( connect(s, (const struct sockaddr *)&skun, sizeof(skun)) == 0)
-	{
-		/* That was easy, we're done. */
-
-		if (fcntl(s, F_SETFL, 0) < 0)
-		{
-			close(s);
-			return (-1);
-		}
-		return (s);
-	}
-
-	if ( errno != EINPROGRESS && errno != EWOULDBLOCK)
-	{
-		perror("authdaemon: connect");
-		close(s);
-		return (-1);
-	}
-
-        /* Wait for the connection to go through, until the timeout expires */
-
-	FD_ZERO(&fdr);
-	FD_SET(s, &fdr);
-	tv.tv_sec=TIMEOUT;
-	tv.tv_usec=0;
-
-	rc=select(s+1, 0, &fdr, 0, &tv);
-        if (rc <= 0 || !FD_ISSET(s, &fdr))
-	{
-		syslog(LOG_CRIT, "authdaemon: connect timed out");
-		close(s);
-		return (-1);
-	}
-
-	if ( connect(s, (const struct sockaddr *)&skun, sizeof(skun)))
-        {
-                if (errno != EISCONN)
-		{
-			syslog(LOG_CRIT, "authdaemon: connect failed: %m");
-			close(s);
-			return (-1);
-		}
-	}
-
-        if (fcntl(s, F_SETFL, 0) < 0)
-	{
-		syslog(LOG_CRIT, "authdaemon: fcntl() failed: %m");
+		syslog(LOG_CRIT, "authdaemon: s_connect() failed: %m");
 		close(s);
 		return (-1);
 	}
@@ -165,6 +113,23 @@ time_t	end_time, curtime;
 	*p=0;
 }
 
+int authdaemondopasswd(char *buffer, int bufsiz)
+{
+	int s=opensock();
+
+	if (s < 0)
+		return (1);
+	if (writeauth(s, buffer, strlen(buffer)))
+	{
+		close(s);
+		return (1);
+	}
+
+	readauth(s, buffer, bufsiz);
+	close(s);
+	return (strcmp(buffer, "OK\n"));
+}
+
 int authdaemondo(const char *authreq,
 	int (*func)(struct authinfo *, void *), void *arg)
 {
