@@ -17,7 +17,7 @@
 #include	"authldap.h"
 #include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authldap.c,v 1.13 2001/04/14 04:43:34 mrsam Exp $";
+static const char rcsid[]="$Id: authldap.c,v 1.14 2001/06/20 03:33:02 mrsam Exp $";
 
 
 struct callback_info {
@@ -260,9 +260,12 @@ extern int auth_ldap_pre(const char *userid, const char *service,
         int (*callback)(struct authinfo *, void *),
 		  void *arg);
 
+extern int auth_ldap_changepw(const char *, const char *, const char *,
+			      const char *);
+
 struct authstaticinfo authldap_info={
 	"authldap",
 	auth_ldap,
 	auth_ldap_pre,
 	authldapclose,
-	0};
+	auth_ldap_changepw};
