@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -75,7 +75,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.66 2001/08/25 21:59:26 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.71 2002/01/08 13:27:18 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -104,6 +104,78 @@ void rfc2045_error(const char *p)
 	_exit(0);
 }
 
+
+extern int maildirsize_read(const char *,int *,off_t *,unsigned *,unsigned *,struct stat *);
+
+void quotainfo_out(const char* qroot)
+{
+	char    quotabuf[QUOTABUFSIZE];
+	char	qresult[200]="";
+	char	qbuf[200];
+	
+	if ((maildir_getquota(".", quotabuf) == 0) && (strcmp(qroot,"SHARED") != 0))
+	{
+	int	quotafd;
+	int	i,quota_s=-1,quota_c=-1;
+	char    *quota=quotabuf;
+	off_t		maildirsize_size=0;
+	unsigned	maildirsize_cnt=0;
+	unsigned	dummy;
+	struct	stat	stat_buf;
+
+		(void) maildir_checkquota(".", &quotafd, quotabuf, 0, 0);
+		if (quotafd >= 0)	close(quotafd);
+
+		if (maildirsize_read("maildirsize",&quotafd,
+			&maildirsize_size,&maildirsize_cnt,&dummy,&stat_buf) == 0)
+			close(quotafd);
+
+		while (quota && *quota)
+		{
+			if (*quota < '0' || *quota > '9')
+			{
+				++quota;
+				continue;
+			}
+
+			i=0;
+			while (*quota >= '0' && *quota <= '9')
+				i=i*10 + (*quota++ - '0');
+			switch (*quota) 
+			{
+				case 'S': quota_s=i; break;
+				case 'C': quota_c=i; break;
+				default : break;
+			}
+		}
+			
+		if (quota_s != -1)
+		{
+			sprintf(qbuf,"STORAGE %d %d",((int)maildirsize_size+1023)/1024,(quota_s+1023)/1024);
+			strcat(qresult,qbuf);
+		}
+		if (quota_c != -1)
+		{
+			sprintf(qbuf,"MESSAGE %d %d",maildirsize_cnt,quota_c);
+			if (strcmp(qresult,"")!=0) strcat(qresult," ");
+			strcat(qresult,qbuf);
+		}
+		
+	}
+
+	writes("* ");
+	writes("QUOTA \"");
+	writes(qroot);
+	writes("\"");
+	if (strcmp(qresult,"")!=0)
+	{
+		writes(" (");
+		writes(qresult);
+		writes(")");
+	};
+	writes("\r\n");
+}
+
 int is_trash(const char *m)
 {
 	if (strcmp(m, "." TRASH))
@@ -188,6 +260,7 @@ void emptytrash()
 	free(all_settings);
 }
 
+#if 0
 int is_draft(const char *m)
 {
 #if 1
@@ -199,6 +272,7 @@ int is_draft(const char *m)
 	return (0);
 #endif
 }
+#endif
 
 int is_reserved(const char *m)
 {
@@ -416,13 +490,14 @@ const char *filename=mi->filename;
 		flags->seen=flags->answered=flags->deleted=flags->flagged
 		=flags->recent=flags->drafts=0;
 
-	if (is_draft(current_mailbox))
+	if ((filename=strrchr(filename, ':')) == 0
+		|| strncmp(filename, ":2,", 3))	return;
+
+	if (strchr(filename, 'D'))
 	{
 		if (buf) strcat(buf, "\\Draft");
 		if (flags)  flags->drafts=1;
 	}
-	if ((filename=strrchr(filename, ':')) == 0
-		|| strncmp(filename, ":2,", 3))	return;
 
 	if (strchr(filename, 'F'))
 	{
@@ -498,6 +573,7 @@ char	*mailbox;
 
 void append_flags(char *buf, struct imapflags *flags)
 {
+	if (flags->drafts)	strcat(buf, "D");
 	if (flags->flagged)	strcat(buf, "F");
 	if (flags->answered)	strcat(buf, "R");
 	if (flags->seen)	strcat(buf, "S");
@@ -909,7 +985,8 @@ unsigned long i, j;
 int	needstats=0;
 unsigned long expunged;
 
-	if (imapscan_maildir(&new_mailbox_info, current_mailbox, 0))
+	if (imapscan_maildir(&new_mailbox_info, current_mailbox, 0,
+	      current_mailbox_ro))
 		return;
 
 	j=0;
@@ -1000,6 +1077,7 @@ const char * envp = getenv("IMAP_IDLE_TIMEOUT");
 const int idleTimeout = envp ? atoi(envp) : 60;
 
        writes("+ entering idle mode\r\n");
+       noop();
        writeflush();
        while (!doidle(idleTimeout))
        {
@@ -1498,6 +1576,58 @@ int	uid=0;
 		return (0);
 	}
 
+	if (strcmp(curtoken->tokenbuf, "GETQUOTAROOT") == 0)
+	{
+	char	*mailbox;
+	char	qroot[20]="ROOT";
+	
+		curtoken=nexttoken_nouc();
+		mailbox=parse_mailbox_error(tag, curtoken, 0, 1);
+		if ( mailbox == 0)
+			return (0);
+		if (is_sharedsubdir(mailbox))
+			strcpy(qroot,"SHARED");
+
+		writes("*");
+		writes(" QUOTAROOT \"");
+		writeqs(mailbox);
+		writes("\" \"");
+		writes(qroot);
+		writes("\"\r\n");
+		quotainfo_out(qroot);
+		writes(tag);
+		writes(" OK GETQUOTAROOT Ok.\r\n");
+		return(0);
+	}
+
+
+	if (strcmp(curtoken->tokenbuf, "SETQUOTA") == 0)
+	{
+		writes(tag);
+		writes(" NO SETQUOTA No permission.\r\n");
+		return(0);
+	}
+
+	if (strcmp(curtoken->tokenbuf, "GETQUOTA") == 0)
+	{
+		curtoken=nexttoken_nouc();
+#if IMAP_CLIENT_BUGS
+		/* allow any quota root name */
+#else
+		if ((strcmp(curtoken->tokenbuf, "ROOT") != 0)&&
+		    (strcmp(curtoken->tokenbuf, "SHARED") != 0))
+		{
+			writes(tag);
+			writes(" BAD GETQUOTA No such Quota Root.\r\n");
+			return(0);
+		}
+#endif
+		quotainfo_out(curtoken->tokenbuf);
+		writes(tag);
+		writes(" OK GETQUOTA Ok.\r\n");
+		return(0);
+	}
+
 	if (strcmp(curtoken->tokenbuf, "STATUS") == 0)
 	{
 	char	*mailbox;
@@ -1566,7 +1696,7 @@ int	uid=0;
 			loaded_infoptr= &other_info;
 			infoptr=loaded_infoptr;
 
-			if (imapscan_maildir(infoptr, mailbox, 1))
+			if (imapscan_maildir(infoptr, mailbox, 1, 1))
 			{
 				writes(tag);
 				writes(" NO [ALERT] STATUS failed\r\n");
@@ -1929,7 +2059,7 @@ int	uid=0;
 			imapscan_free(&current_mailbox_info);
 			current_mailbox=0;
 		}
-		if (imapscan_maildir(&current_mailbox_info, mailbox, 0))
+		if (imapscan_maildir(&current_mailbox_info, mailbox, 0, ro))
 		{
 			free(mailbox);
 			writes(tag);
@@ -1946,10 +2076,8 @@ int	uid=0;
 
 		current_mailbox_ro=ro;
 
-		writes("* FLAGS (");
-		if (is_draft(current_mailbox))
-			writes("\\Draft ");
-		writes("\\Answered \\Flagged \\Deleted \\Seen \\Recent)\r\n");
+		writes("* FLAGS (\\Draft \\Answered \\Flagged"
+		       " \\Deleted \\Seen \\Recent)\r\n");
 		writes("* OK [PERMANENTFLAGS (");
 
 
@@ -1959,9 +2087,7 @@ int	uid=0;
 		}
 		else
 		{
-			if (is_draft(current_mailbox))
-				writes("\\Draft ");
-			writes("\\Answered \\Flagged \\Deleted \\Seen)] Limited\r\n");
+			writes("\\Draft \\Answered \\Flagged \\Deleted \\Seen)] Limited\r\n");
 		}
 		mailboxmetrics();
 		writes("* OK [UIDVALIDITY ");
@@ -2234,7 +2360,7 @@ int	uid=0;
 	struct searchinfo *si, *sihead;
 	unsigned long i;
 
-		curtoken=nexttoken();
+		curtoken=nexttoken_okbracket();
 		if (curtoken->tokentype == IT_ATOM &&
 			strcmp(curtoken->tokenbuf, "CHARSET") == 0)
 		{
