@@ -26,7 +26,7 @@
 #include	"authlib/authmod.h"
 #include	"tcpd/spipe.h"
 
-static const char rcsid[]="$Id: imaplogin.c,v 1.9 2001/02/20 13:01:05 mrsam Exp $";
+static const char rcsid[]="$Id: imaplogin.c,v 1.11 2001/11/29 04:25:56 mrsam Exp $";
 
 FILE *debugfile=0;
 extern void mainloop();
@@ -119,7 +119,8 @@ int	waitstat;
 
 int do_imap_command(const char *tag)
 {
-struct	imaptoken *curtoken=nexttoken();
+	struct	imaptoken *curtoken=nexttoken();
+	char authservice[40];
 
 	if (strcmp(curtoken->tokenbuf, "LOGOUT") == 0)
 	{
@@ -164,6 +165,7 @@ struct	imaptoken *curtoken=nexttoken();
 	char	*userid;
 	char	*tagenv;
 	char	*passwd;
+	const char *p;
 
 		if (have_starttls() && tlsrequired())	/* Not yet */
 		{
@@ -208,8 +210,17 @@ struct	imaptoken *curtoken=nexttoken();
 			return (-1);
 		}
 		putenv(tagenv);
-		authmod_login(main_argc-1, main_argv+1, "imap",
-			userid, passwd);
+
+		strcat(strcpy(authservice, "AUTHSERVICE"),
+		       getenv("TCPLOCALPORT"));
+
+		p=getenv(authservice);
+
+		if (!p || !*p)
+			p="imap";
+
+		authmod_login(main_argc-1, main_argv+1, p,
+                         userid, passwd);
 	}
 
 	if (strcmp(curtoken->tokenbuf, "AUTHENTICATE") == 0)
@@ -220,7 +231,11 @@ struct	imaptoken *curtoken=nexttoken();
 			writes(" NO STARTTLS required\r\n");
 			return (0);
 		}
-		return (authenticate(tag));
+		authenticate(tag);
+		writes(tag);
+		writes(" NO Authentication failed.\r\n");
+		writeflush();
+		return (-2);
 	}
 
 	return (-1);
