@@ -1,11 +1,14 @@
 /*
-** Copyright 2000 Double Precision, Inc.
+** Copyright 2000-2001 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
 #include	"maildirfilter.h"
 #include	"maildirfiltertypelist.h"
 #include	"maildirgetquota.h"
+#include	"mailbot.h"
+#include	"autoresponse.h"
+#include	"numlib/numlib.h"
 #include	<stdlib.h>
 #include	<string.h>
 #include	<stdio.h>
@@ -19,16 +22,17 @@
 #include	<unistd.h>
 #endif
 
-static const char rcsid[]="$Id: maildirfilter.c,v 1.4 2000/12/02 05:38:53 mrsam Exp $";
+static const char rcsid[]="$Id: maildirfilter.c,v 1.12 2002/01/08 04:47:58 mrsam Exp $";
 
 struct maildirfilterrule *mf_appendrule(struct maildirfilter *r,
-	const char *name,
-	enum maildirfiltertype type,
-	int flags,
-	const char *header,
-	const char *value,
-	const char *folder,
-	int *errcode)
+					const char *name,
+					enum maildirfiltertype type,
+					int flags,
+					const char *header,
+					const char *value,
+					const char *folder,
+					const char *fromhdr,
+					int *errcode)
 {
 struct maildirfilterrule *p=malloc(sizeof(struct maildirfilterrule));
 
@@ -44,7 +48,7 @@ struct maildirfilterrule *p=malloc(sizeof(struct maildirfilterrule));
 	r->last=p;
 
 	if (mf_ruleupdate(r, p, name, type, flags,
-			header, value, folder, errcode))
+			  header, value, folder, fromhdr, errcode))
 	{
 		mf_ruledel(r, p);
 		return (0);
@@ -53,14 +57,15 @@ struct maildirfilterrule *p=malloc(sizeof(struct maildirfilterrule));
 }
 
 int mf_ruleupdate(struct maildirfilter *r,
-	struct maildirfilterrule *p,
-	const char *name,
-	enum maildirfiltertype type,
-	int flags,
-	const char *header,
-	const char *value,
-	const char *folder,
-	int *errcode)
+		  struct maildirfilterrule *p,
+		  const char *name,
+		  enum maildirfiltertype type,
+		  int flags,
+		  const char *header,
+		  const char *value,
+		  const char *folder,
+		  const char *fromhdr,
+		  int *errcode)
 {
 const char *c;
 
@@ -148,7 +153,7 @@ const char *c;
 			return (-1);
 		}
 	}
-	else
+	else if (!(flags & MFR_PLAINSTRING))
 	{
 /*
 ** This is ugly.  We: make sure parenthesis match up, and look out for
@@ -159,7 +164,7 @@ const char *c;
 		while (*c)
 		{
 			if (*c == '/' || *c == '$' || *c == '!'
-				|| *c == '`'
+				|| *c == '`' || (int)(unsigned char)*c < ' '
 				|| *c == '\'' || *c == '"') return (-1);
 						/* must be escaped */
 
@@ -197,6 +202,8 @@ const char *c;
 					if (*c == '\\')
 						++c;
 					if (!*c)	return (-1);
+					if ((int)(unsigned char)*c < ' ')
+						return (-1);
 					++c;
 					if (*c == ']')	break;
 					if (*c != '-')	continue;
@@ -207,6 +214,8 @@ const char *c;
 						return (-1); /* must be quoted*/
 					if (*c == '\\')
 						++c;
+					if ((int)(unsigned char)*c < ' ')
+						return (-1);
 					if (!*c)	return (-1);
 					++c;
 				}
@@ -225,6 +234,18 @@ const char *c;
 		if (lparencount)	return (-1);	/* not balanced */
 	}
 
+	/* validate FROM header */
+
+	*errcode=MF_ERR_BADFROMHDR;
+
+	while (fromhdr && *fromhdr && isspace((int)(unsigned char)*fromhdr))
+		++fromhdr;
+
+	for (c=fromhdr; *c; c++)
+		if (*c == '\'' || *c == '\\' ||
+		    (int)(unsigned char)*c < ' ')
+			return (-1);
+
 	*errcode=MF_ERR_BADRULEFOLDER;
 
 	/* validate name of destination folder */
@@ -235,6 +256,8 @@ const char *c;
 
 	if (*c == '*' || *c == '!')
 	{
+		/* Forward, or bounce with an error */
+
 		++c;
 		for ( ; *c; c++)
 		{
@@ -243,6 +266,15 @@ const char *c;
 				return (-1);
 		}
 	}
+	else if (*c == '+')	/* Autorespond */
+	{
+		struct mf_autoresp_info ai;
+
+		if (mf_autoresp_info_init_str(&ai, c+1))
+			return (-1);
+
+		mf_autoresp_info_free(&ai);
+	}
 	else if (strcmp(c, "."))
 	{
 		for ( ; *c; c++)
@@ -257,6 +289,8 @@ const char *c;
 
 	/* OK, we're good */
 
+	*errcode=MF_ERR_INTERNAL;
+
 	if (p->rulename)	free(p->rulename);
 	if ((p->rulename=strdup(name)) == 0)	return (-1);
 	p->type=type;
@@ -268,9 +302,15 @@ const char *c;
 	if ((p->tofolder=malloc(strlen(folder)+2)) == 0)	return (-1);
 	*p->tofolder=0;
 	/* Prefix with dot by default */
-	if (*folder != '*' && *folder != '!' && *folder != '.')
+	if (*folder != '*' && *folder != '!' && *folder != '.'
+	    && *folder != '+')
 		strcpy(p->tofolder, ".");
 	strcat(p->tofolder, folder);
+
+	if (p->fromhdr)		free(p->fromhdr);
+	if ((p->fromhdr=strdup(fromhdr ? fromhdr:"")) == NULL)
+		return (-1);
+
 	p->flags=flags;
 	return (0);
 }
@@ -287,6 +327,7 @@ void mf_ruledel(struct maildirfilter *r, struct maildirfilterrule *p)
 	if (p->fieldname)	free(p->fieldname);
 	if (p->fieldvalue)	free(p->fieldvalue);
 	if (p->tofolder)	free(p->tofolder);
+	if (p->fromhdr)		free(p->fromhdr);
 	free(p);
 }
 
@@ -324,6 +365,24 @@ struct maildirfilterrule *q;
 	q->next=p;
 }
 
+static void print_pattern(FILE *f, int flags, const char *v)
+{
+	if (!(flags & MFR_PLAINSTRING))
+	{
+		fprintf(f, "%s%s",
+			*v && isspace((int)(unsigned char)*v) ? "\\":"", v);
+		return;
+	}
+
+	while (*v)
+	{
+		if (!isalnum((int)(unsigned char)*v))
+			putc('\\', f);
+		putc((int)(unsigned char)*v, f);
+		++v;
+	}
+}
+
 int mf_saverules(struct maildirfilter *r, const char *filename,
 		 const char *maildir,
 		 const char *maildirpath, const char *fromaddr)
@@ -334,7 +393,7 @@ char quotabuf[QUOTABUFSIZE];
 
 	if (!f)	return (-1);
 
-	fprintf(f,	"#MFMAILDROP=1\n"
+	fprintf(f,	"#MFMAILDROP=2\n"
 			"#\n"
 			"# DO NOT EDIT THIS FILE.  This is an automatically"
 						" generated filter.\n"
@@ -366,7 +425,7 @@ char quotabuf[QUOTABUFSIZE];
 			putc(*fromaddr, f);
 	}
 	fprintf(f, "\'\n\n");
-	
+
 	for (p=r->first; p; p=p->next)
 	{
 	const char *fieldname=p->fieldname ? p->fieldname:"";
@@ -378,7 +437,10 @@ char quotabuf[QUOTABUFSIZE];
 		fprintf(f, "##Header:%s\n", fieldname);
 		fprintf(f, "##Value:%s\n", fieldvalue);
 		fprintf(f, "##Folder:%s\n", tofolder);
+		fprintf(f, "##From:%s\n", p->fromhdr ? p->fromhdr:"");
 
+		if (p->flags & MFR_PLAINSTRING)
+			fprintf(f, "##PlainString\n");
 		if (p->flags & MFR_DOESNOT)
 			fprintf(f, "##DoesNot\n");
 		if (p->flags & MFR_BODY)
@@ -397,21 +459,45 @@ char quotabuf[QUOTABUFSIZE];
 		switch (p->type)	{
 		case startswith:
 			if (p->flags & MFR_BODY)
-				fprintf(f, "/^%s/:b", fieldvalue);
+			{
+				fprintf(f, "/^");
+				print_pattern(f, p->flags, fieldvalue);
+				fprintf(f, "/:b");
+			}
 			else
-				fprintf(f, "/^%s: *%s/", fieldname, fieldvalue);
+			{
+				fprintf(f, "/^%s: *", fieldname);
+				print_pattern(f, p->flags, fieldvalue);
+				fprintf(f, "/");
+			}
 			break;
 		case endswith:
 			if (p->flags & MFR_BODY)
-				fprintf(f, "/%s$/:b", fieldvalue);
+			{
+				fprintf(f, "/");
+				print_pattern(f, p->flags, fieldvalue);
+				fprintf(f, "$/:b");
+			}
 			else
-				fprintf(f, "/^%s:.*%s$/", fieldname,fieldvalue);
+			{
+				fprintf(f, "/^%s:.*", fieldname);
+				print_pattern(f, p->flags, fieldvalue);
+				fprintf(f, "$/");
+			}
 			break;
 		case contains:
 			if (p->flags & MFR_BODY)
-				fprintf(f, "/%s/:b", fieldvalue);
+			{
+				fprintf(f, "/");
+				print_pattern(f, p->flags, fieldvalue);
+				fprintf(f, "/:b");
+			}
 			else
-				fprintf(f, "/^%s:.*%s/", fieldname,fieldvalue);
+			{
+				fprintf(f, "/^%s:.*", fieldname);
+				print_pattern(f, p->flags, fieldvalue);
+				fprintf(f, "/");
+			}
 			break;
 		case hasrecipient:
 			fprintf(f, "hasaddr(\"%s\")", fieldvalue);
@@ -433,7 +519,7 @@ char quotabuf[QUOTABUFSIZE];
 
 		if (*tofolder == '!')
 		{
-			fprintf(f, "    %s \"| $SENDMAIL -f '$FROM' %s\"\n",
+			fprintf(f, "    %s \"| $SENDMAIL -t -f '$FROM' %s\"\n",
 				p->flags & MFR_CONTINUE ? "cc":"to",
 					tofolder+1);
 		}
@@ -443,6 +529,36 @@ char quotabuf[QUOTABUFSIZE];
 				"    EXITCODE=77\n"
 				"    exit\n", tofolder+1);
 		}
+		else if (*tofolder == '+')
+		{
+			struct mf_autoresp_info ai;
+
+			if (mf_autoresp_info_init_str(&ai, tofolder+1) == 0)
+			{
+
+				if (p->fromhdr && p->fromhdr[0])
+					fprintf(f, "    AUTOREPLYFROM='%s'\n",
+						p->fromhdr);
+				else
+					fprintf(f, "    AUTOREPLYFROM=$FROM\n"
+						);
+
+				fprintf(f, "   `%s -A \"X-Sender: $FROM\""
+					" -A \"From: $AUTOREPLYFROM\"",
+					MAILBOT);
+				if (ai.dsnflag)
+					fprintf(f, " -M \"$FROM\"");
+				fprintf(f, " -m \"%s/autoresponses/%s\"",
+					maildirpath, ai.name);
+				if (ai.days > 0)
+					fprintf(f,
+						" -d \"%s/autoresponses/"
+						"%s.dat\" -D %u",
+					maildirpath, ai.name, ai.days);
+				fprintf(f, " $SENDMAIL -t -f \"\"`\n");
+				mf_autoresp_info_free(&ai);
+			}
+		}
 		else
 		{
 			if (*tofolder == '.')	++tofolder;
@@ -482,12 +598,21 @@ enum	maildirfiltertype new_type;
 char	new_header[256];
 char	new_value[256];
 char	new_folder[256];
+char	new_autoreplyfrom[512];
+
 int	flags;
 
 	if (!f)	return (MF_LOADNOTFOUND);
 
 	if (fgets(buf, sizeof(buf), f) == 0 ||
-		strcmp(buf, "#MFMAILDROP=1\n"))
+		strncmp(buf, "#MFMAILDROP=", 12))
+	{
+		fclose(f);
+		return (MF_LOADFOREIGN);
+	}
+
+	flags=atoi(buf+12);
+	if (flags != 1 && flags != 2)
 	{
 		fclose(f);
 		return (MF_LOADFOREIGN);
@@ -497,6 +622,7 @@ int	flags;
 	new_header[0]=0;
 	new_value[0]=0;
 	new_folder[0]=0;
+	new_autoreplyfrom[0]=0;
 	flags=0;
 
 #define	SET(f,b) { f[0]=0; strncat( (f), (b), sizeof(f)-1); }
@@ -512,6 +638,14 @@ int	flags;
 		while ( *p && isspace((int)(unsigned char)*p))
 			++p;
 
+		if (strncasecmp(p, "From:", 5) == 0)
+		{
+			p += 5;
+			SET(new_autoreplyfrom, p);
+			continue;
+		}
+
+
 		if (strncasecmp(p, "Op:", 3) == 0)
 		{
 			p += 3;
@@ -549,6 +683,12 @@ int	flags;
 			continue;
 		}
 
+		if (strcasecmp(p, "plainstring") == 0)
+		{
+			flags |= MFR_PLAINSTRING;
+			continue;
+		}
+
 		if (strcasecmp(p, "doesnot") == 0)
 		{
 			flags |= MFR_DOESNOT;
@@ -573,8 +713,10 @@ int	flags;
 		struct maildirfilterrule *rr;
 
 			p += 5;
-			if ((rr=mf_appendrule(r, p, new_type, flags, new_header,
-				new_value, new_folder, &dummy)) == 0)
+			if ((rr=mf_appendrule(r, p, new_type, flags,
+					      new_header,
+					      new_value, new_folder,
+					      new_autoreplyfrom, &dummy)) == 0)
 			{
 				fclose(f);
 				return (MF_LOADERROR);
@@ -583,9 +725,83 @@ int	flags;
 			new_header[0]=0;
 			new_value[0]=0;
 			new_folder[0]=0;
+			new_autoreplyfrom[0]=0;
 			flags=0;
 		}
 	}
 	fclose(f);
 	return (MF_LOADOK);
 }
+
+int mf_autoresp_info_init(struct mf_autoresp_info *i, const char *c)
+{
+	memset(i, 0, sizeof(*i));
+
+	if (autoresponse_validate(NULL, c))
+		return (-1);
+	i->name=strdup(c);
+	if (!(i->name))
+		return (-1);
+	return (0);
+}
+
+int mf_autoresp_info_init_str(struct mf_autoresp_info *i, const char *c)
+{
+	char *p;
+
+	memset(i, 0, sizeof(*i));
+	i->name=strdup(c);
+	if (!(i->name))
+		return (-1);
+
+	if (strtok(i->name, " \t\r\n") == NULL)
+	{
+		errno=EINVAL;
+		free(i->name);
+		i->name=0;
+		return (-1);
+	}
+
+	while ((p=strtok(NULL, " \t\r\n")) != NULL)
+	{
+		if (strncmp(p, "dsn=", 4) == 0)
+			i->dsnflag=atoi(p+4) ? 1:0;
+		else if (strncmp(p, "days=", 5) == 0)
+			i->days=atoi(p+5);
+	}
+	return (0);
+}
+
+void mf_autoresp_info_free(struct mf_autoresp_info *i)
+{
+	if (i->name)
+	{
+		free(i->name);
+		i->name=0;
+	}
+}
+
+char *mf_autoresp_info_asstr(struct mf_autoresp_info *i)
+{
+	char days_buf[NUMBUFSIZE+10];
+
+	const char *dsn_arg="";
+	const char *days_arg="";
+	char *p;
+
+	if (i->dsnflag)
+		dsn_arg=" dsn=1";
+	if (i->days > 0)
+	{
+		strcpy(days_buf, " days=");
+		str_size_t(i->days, days_buf+6);
+		days_arg=days_buf;
+	}
+
+	p=malloc(strlen(i->name)+1+strlen(dsn_arg)+strlen(days_arg));
+	if (!p)
+		return (NULL);
+
+	strcat(strcat(strcpy(p, i->name), dsn_arg), days_arg);
+	return (p);
+}
