@@ -6,6 +6,10 @@
 #include	"argparse.h"
 #include	"spipe.h"
 #include	"rfc1035/rfc1035.h"
+#include	"soxwrap/soxwrap.h"
+#ifdef  getc
+#undef  getc
+#endif
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
@@ -48,7 +52,7 @@
 #include	<openssl/err.h>
 #include	<sys/time.h>
 
-static const char rcsid[]="$Id: starttls.c,v 1.20 2001/04/18 01:48:46 mrsam Exp $";
+static const char rcsid[]="$Id: starttls.c,v 1.21 2001/09/20 21:44:40 mrsam Exp $";
 
 #ifndef NO_RSA
 static RSA *rsa_callback(SSL *, int, int);
@@ -782,7 +786,7 @@ int	port_num;
 		if (rfc1035_mkaddress(af, &addrbuf, addrs+n, port_num,
 			&saddr, &saddrlen))	continue;
 
-		if (connect(fd, saddr, saddrlen) == 0)
+		if (sox_connect(fd, saddr, saddrlen) == 0)
 			break;
 	}
 	free(addrs);
