@@ -56,7 +56,7 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.23 2001/06/13 21:42:44 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.27 2001/11/22 15:29:38 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *listenarg=0;
@@ -64,6 +64,7 @@ static const char *ipaddrarg=0;
 static const char *userarg=0;
 static const char *grouparg=0;
 static const char *maxprocsarg=0;
+static const char *warnarg=0;
 static const char *maxperiparg=0;
 static const char *maxpercarg=0;
 static const char *nodnslookup=0;
@@ -89,6 +90,7 @@ static struct args arginfo[]={
 	{"maxperc", &maxpercarg},
 	{"maxperip", &maxperiparg},
 	{"maxprocs", &maxprocsarg},
+	{"warn", &warnarg},
 	{"nodnslookup", &nodnslookup},
 	{"noidentlookup", &noidentlookup},
 	{"pid", &pidarg},
@@ -113,8 +115,9 @@ static struct portinfo {
 } *fdlist=0;
 static int maxfd;
 
-static int nprocs, maxperc, maxperip;
+static int nprocs, maxperc, maxperip, nwarn;
 static pid_t *pids;
+static time_t last_alert=0, last_warn=0;
 static RFC1035_ADDR *addrs;
 
 static int sighup_received=0;
@@ -716,6 +719,16 @@ int	lockfd;
 		}
 	}
 
+	nwarn= nprocs - (nprocs / 10 + 1);
+
+	if (warnarg)
+	{
+		int c=atoi(warnarg);
+
+		if (c >= 0 && c <= nprocs)
+			nwarn=c;
+	}
+
 	if ((pids=malloc(sizeof(*pids)*nprocs)) == 0)
 	{
 		perror("malloc");
@@ -1027,7 +1040,7 @@ static int doit(int argn, int argc, char **argv)
 		{
 			if (pi->fd1 >= 0 && FD_ISSET(pi->fd1, &fdr) &&
 			    ((n=getfreeslot(&pidptr)),
-			     (sinl = sizeof(struct sockaddr)),
+			     (sinl = sizeof(sin)),
 			     (sockfd=sox_accept(pi->fd1,
 						(struct sockaddr *)&sin,
 						&sinl))) >= 0)
@@ -1038,7 +1051,7 @@ static int doit(int argn, int argc, char **argv)
 
 			if (pi->fd2 >= 0 && FD_ISSET(pi->fd2, &fdr) &&
 			    ((n=getfreeslot(&pidptr)),
-			     (sinl = sizeof(struct sockaddr)), 
+			     (sinl = sizeof(sin)), 
 			     (sockfd=sox_accept(pi->fd2,
 						(struct sockaddr *)&sin,
 						&sinl))) >= 0)
@@ -1061,6 +1074,7 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 	struct	linger l;
 #endif
 	pid_t	p;
+	int cnt;
 
 	if (rfc1035_sockaddrip(sin, sinl, &addr)
 	    || rfc1035_sockaddrport(sin, sinl, &addrport))
@@ -1140,6 +1154,36 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 	memcpy(addrs+n, &addr, sizeof(addr));
 	sox_close(sockfd);
 	wait_clear(childsig);
+
+	for (cnt=n=0; n<nprocs; n++)
+		if (pids[n] != (pid_t)-1)
+			++cnt;
+
+	if (cnt == nprocs)
+	{
+		time_t t;
+
+		time(&t);
+		if (last_alert == 0 || last_alert > t || last_alert < t - 60)
+		{
+			last_alert=t;
+			fprintf(stderr,
+				"ALERT: %d maximum active connections.\n",
+				nprocs);
+		}
+	}
+	else if (cnt >= nwarn)
+	{
+		time_t t;
+
+		time(&t);
+		if (last_warn == 0 || last_warn > t || last_warn < t - 60)
+		{
+			last_warn=t;
+			fprintf(stderr, "WARN: %d active connections.\n",
+				cnt);
+		}
+	}
 }
 
 static void mysetenv(const char *name, const char *val)
@@ -1315,7 +1359,7 @@ int i;
 
 	if (!varname)	varname="BLOCK";
 
-	if ((q=getenv(varname)) != 0 && *q)	return;
+	if ((q=getenv(varname)) != 0)	return;
 					/* Env var already set */
 
 	if (p->ia.s_addr == INADDR_ANY)
@@ -1572,6 +1616,7 @@ const char *remoteinfo;
 	}
 	proxy();
 	execv(prog, argv);
+	perror(prog);
 	exit(1);
 }
 
