@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2000 Double Precision, Inc.
+** Copyright 1998 - 2001 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -7,17 +7,8 @@
 #include	"config.h"
 #endif
 #include	"tcpremoteinfo.h"
-#if TIME_WITH_SYS_TIME
-#include	<sys/time.h>
-#include	<time.h>
-#else
-#if HAVE_SYS_TIME_H
-#include	<sys/time.h>
-#else
-#include	<time.h>
-#endif
-#endif
-#include	<sys/types.h>
+#include	"soxwrap/sconnect.h"
+
 #if HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
@@ -30,7 +21,7 @@
 
 #include	"soxwrap/soxwrap.h"
 
-static const char rcsid[]="$Id: tcpremoteinfo.c,v 1.4 2000/05/21 20:27:56 mrsam Exp $";
+static const char rcsid[]="$Id: tcpremoteinfo.c,v 1.6 2001/12/08 21:24:50 mrsam Exp $";
 
 const char *tcpremoteinfo(const RFC1035_ADDR *laddr, int lport,
 	const RFC1035_ADDR *raddr, int rport, const char **ostype)
@@ -63,63 +54,21 @@ int	addrlen;
 		return (0);
 	}
 
-#ifdef	O_NONBLOCK
-	if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
-#else
-	if (fcntl(fd, F_SETFL, O_NDELAY) < 0)
-#endif
+	time (&current_time);
+	max_time=current_time+30;
+
+	if (rfc1035_mkaddress(n, &sin, raddr, htons(113), &addr, &addrlen) < 0)
 	{
 		sox_close(fd);
 		return (0);
 	}
-	time (&current_time);
-	max_time=current_time+30;
 
-#ifdef  SOL_LINGER
+	if (s_connect(fd, addr, addrlen, max_time - current_time, 1) < 0)
 	{
-	struct linger l;
-
-                l.l_onoff=0;
-                l.l_linger=0;
-
-                setsockopt(fd, SOL_SOCKET, SOL_LINGER,
-			(const char *)&l, sizeof(l));
-	}
-#endif
-
-	if (rfc1035_mkaddress(n, &sin, raddr, htons(113), &addr, &addrlen) < 0)
-	{
-		close(fd);
+		sox_close(fd);
 		return (0);
 	}
 
-	if (sox_connect(fd, addr, addrlen) < 0)
-	{
-		if (errno != EINPROGRESS)
-		{
-			sox_close(fd);
-			return (0);
-		}
-		FD_ZERO(&fds);
-		FD_SET(fd, &fds);
-		tv.tv_sec=max_time-current_time;
-		tv.tv_usec=0;
-		if (sox_select(fd+1, 0, &fds, 0, &tv) != 1 ||
-			!FD_ISSET(fd, &fds))
-		{
-			sox_close(fd);
-			return (0);
-		}
-		if (sox_connect(fd, addr, addrlen) < 0)
-		{
-			if (errno != EISCONN)
-			{
-				sox_close(fd);
-				return (0);
-			}
-		}
-	}
-
 	sprintf(buf, "%d,%d\r\n", ntohs(rport), ntohs(lport));
 	bufptr=buf;
 	bufleft=strlen(buf);
